 package ihm;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.undo.*;
import javax.swing.text.*;
import applicatif.*;
import java.net.*;
import java.beans.*;

/**
 * Cette classe reprsente une fentre qui permet d'diter un texte. 
 */
public class FenetreEditeur extends JDialog {

  // Constantes
  private static final String GRAS_TEXT = "Gras";
  private static final String ITAL_TEXT = "Italique";
  private static final String SOULIGNE_TEXT = "Soulign";
  private static final String COUPER_TEXT = "Couper";
  private static final String COPIER_TEXT= "Copier";
  private static final String COLLER_TEXT = "Coller";
  private static final String ICONE_TEXT_PATH = "toolbarButtonGraphics/text/";
  private static final String ICONE_GENERAL_PATH = "toolbarButtonGraphics/general/";
  public static final String SELECTIONNE = "SELECTIONNE";

  // Variables liees a l'IHM
  protected JFrame fenetrePpale;
  protected JToolBar toolBar = new JToolBar();
  protected BorderLayout layout = new BorderLayout();
  protected JTextPane textPane = new JTextPane();
  protected JScrollPane scrollPane = new JScrollPane(textPane);
  protected JPanel panneauBoutons = new JPanel();
  protected JButton btOk = new JButton("OK");
  protected JButton btAnnuler = new JButton("Annuler");

  // Les boutons et menus
  protected JToggleButton boutonGras, boutonItalique, boutonSouligne;
  protected JButton boutonCouper, boutonCopier, boutonColler;
  protected JButton boutonUndo, boutonRedo;
  protected JMenuItem mCouper, mCopier, mColler, mUndo, mRedo;
  protected JMenu mEdition;
  protected JMenu mAnnulation;
  protected JPopupMenu popup;

  // Les actions
  protected Action actionGras, actionItalique, actionSouligne;
  protected Action actionCouper, actionCopier, actionColler;
  protected UndoAction actionUndo;
  protected RedoAction actionRedo;

  // Gestion du undo
  protected UndoManager gestionnaireUndo;

  // Le document  diter
  protected DefaultStyledDocument document;


  /**
   * Constructeur
   */
  public FenetreEditeur(JFrame fen, DefaultStyledDocument doc) {
    super(fen, "Editeur de texte", true);
    //setSize(new Dimension(300, 200));
    fenetrePpale = fen;
    document = doc;
    gererAnnulation();
    initialiserActions();
    initialiserIHM();
    gererMajBoutons();
    scrollPane.setPreferredSize(new Dimension(100, 300));
    pack();
  }

  public void gererAnnulation() {
    textPane.setDocument(document);
    gestionnaireUndo = new UndoManager();
    document.addUndoableEditListener(new EvenementAnnulableListener());

    actionUndo = new UndoAction(gestionnaireUndo);
    actionRedo = new RedoAction(gestionnaireUndo);
    // initialisation des liens inverses
    actionUndo.setRedoAction(actionRedo);
    actionRedo.setUndoAction(actionUndo);
  }

  /**
   * Initialisation des actions
   */
  public void initialiserActions() {
    // Gestion du style
    actionGras = new StyledEditorKit.BoldAction();
    actionGras.putValue(Action.NAME, GRAS_TEXT);
    actionGras.putValue(Action.SMALL_ICON, ihm.GestionnaireIcone.getIcone(ICONE_TEXT_PATH, "Bold16.gif"));

    actionItalique = new StyledEditorKit.ItalicAction();
    actionItalique.putValue(Action.NAME, ITAL_TEXT);
    actionItalique.putValue(Action.SMALL_ICON, ihm.GestionnaireIcone.getIcone(ICONE_TEXT_PATH, "Italic16.gif"));

    actionSouligne = new StyledEditorKit.UnderlineAction();
    actionSouligne.putValue(Action.NAME, SOULIGNE_TEXT);
    actionSouligne.putValue(Action.SMALL_ICON, ihm.GestionnaireIcone.getIcone(ICONE_TEXT_PATH, "Underline16.gif"));

    // Gestion du "couper-copier-coller"
    actionCouper = new DefaultEditorKit.CutAction();
    actionCouper.putValue(Action.NAME, COUPER_TEXT);
    actionCouper.putValue(Action.SMALL_ICON, ihm.GestionnaireIcone.getIcone(ICONE_GENERAL_PATH, "Cut16.gif"));

    actionCopier = new DefaultEditorKit.CopyAction();
    actionCopier.putValue(Action.NAME, COPIER_TEXT);
    actionCopier.putValue(Action.SMALL_ICON, ihm.GestionnaireIcone.getIcone(ICONE_GENERAL_PATH, "Copy16.gif"));

    actionColler = new DefaultEditorKit.PasteAction();
    actionColler.putValue(Action.NAME, COLLER_TEXT);
    actionColler.putValue(Action.SMALL_ICON, ihm.GestionnaireIcone.getIcone(ICONE_GENERAL_PATH, "Paste16.gif"));
  }

  /**
   * Initialisation de l'IHM
   */
  public void initialiserIHM() {
    getContentPane().setLayout(layout);
    creerBarreOutils();
    creerMenuContextuel();
    getContentPane().add(toolBar, BorderLayout.NORTH);
    toolBar.setFloatable(false);
    toolBar.setOrientation(SwingConstants.HORIZONTAL);
    scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    getContentPane().add(scrollPane, BorderLayout.CENTER);
    panneauBoutons.add(btOk);
    panneauBoutons.add(btAnnuler);
    btOk.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // Enregistrement des changements
        ((FenetrePrincipale)fenetrePpale).getPanneauDetail().setContenuDescription(document/*textPane.getText()*/);
        dispose();
      }
    });
    btAnnuler.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dispose();
      }
    });
    getContentPane().add(panneauBoutons, BorderLayout.SOUTH);
  }

  /**
   * Cration de la barre d'outils
   */
  protected void creerBarreOutils() {
    // Cration des boutons
    boutonGras = new JToggleButton(actionGras);
    boutonItalique = new JToggleButton(actionItalique);
    boutonSouligne = new JToggleButton(actionSouligne);
    boutonCouper = new JButton(actionCouper);
    boutonCopier = new JButton(actionCopier);
    boutonColler = new JButton(actionColler);
    boutonUndo = new JButton(actionUndo);
    boutonRedo = new JButton(actionRedo);

    // Pas de texte sur les boutons
    boutonGras.setText("");
    boutonItalique.setText("");
    boutonSouligne.setText("");
    boutonCouper.setText("");
    boutonCopier.setText("");
    boutonColler.setText("");
    boutonUndo.setText("");
    boutonRedo.setText("");

    // Ajout d'un espace entre l'icone et le bord du bouton
    boutonGras.setMargin(new Insets(2,6,2,6));
    boutonItalique.setMargin(new Insets(2,6,2,6));
    boutonSouligne.setMargin(new Insets(2,6,2,6));
    boutonCouper.setMargin(new Insets(2,6,2,6));
    boutonCopier.setMargin(new Insets(2,6,2,6));
    boutonColler.setMargin(new Insets(2,6,2,6));
    boutonUndo.setMargin(new Insets(2,6,2,6));
    boutonRedo.setMargin(new Insets(2,6,2,6));


    // ajout des boutons dans la barre
    toolBar.add(boutonGras);
    toolBar.add(boutonItalique);
    toolBar.add(boutonSouligne);
    toolBar.addSeparator();
    toolBar.add(boutonCouper);
    toolBar.add(boutonCopier);
    toolBar.add(boutonColler);
    toolBar.addSeparator();
    toolBar.add(boutonUndo);
    toolBar.add(boutonRedo);
  }

  /**
   * Cration du menu contextuel
   */
  protected void creerMenuContextuel() {
    mCouper = new JMenuItem(actionCouper);
    mCopier = new JMenuItem(actionCopier);
    mColler = new JMenuItem(actionColler);
    mUndo = new JMenuItem(actionUndo);
    mRedo = new JMenuItem(actionRedo);
    mEdition = new JMenu("Edition");
    mAnnulation = new JMenu("Annulation");
    popup = new JPopupMenu();
    mEdition.add(mCouper);
    mEdition.add(mCopier);
    mEdition.add(mColler);
    mAnnulation.add(mUndo);
    mAnnulation.add(mRedo);
    popup.add(mEdition);
    popup.add(mAnnulation);
    MouseListener popupListener = new PopupListener();
    textPane.addMouseListener(popupListener);
  }

  /**
   * Mthode qui gre le fait que l'tat des boutons concernant le
   * style (gras, italique et soulign) soit mis  jour lorsque
   * le curseur du texte est positionn sur une zone tant en gras,
   * en italique ou en soulign.
   */
  public void gererMajBoutons() {
    // Lorsque le curseur est dplac, les actions reoivent
    // une notification de leur proprit "SELECTIONNE"
    textPane.addCaretListener(new CurseurTexteListener());
    // Il faut abonner les 3 actions  un listener qui coute
    // les vnements de type PropertyChangeEvent et qui change
    // l'tat des boutons
    actionGras.addPropertyChangeListener(new SelectionneListener(boutonGras));
    actionItalique.addPropertyChangeListener(new SelectionneListener(boutonItalique));
    actionSouligne.addPropertyChangeListener(new SelectionneListener(boutonSouligne));
  }

  /**
   * Classe incluse : listener sur les actions annulables
   */
  class EvenementAnnulableListener implements UndoableEditListener {
    public void undoableEditHappened(UndoableEditEvent e) {
      gestionnaireUndo.addEdit(e.getEdit());
      actionUndo.majEtat();
      actionRedo.majEtat();
    }
  }

  /**
   * Classe incluse : mouse listener qui fait apparatre le
   * menu contextuel.
   */
  class PopupListener extends MouseAdapter {
    public void mousePressed(MouseEvent e) {
      afficherPopup(e);
    }

    public void mouseReleased(MouseEvent e) {
      afficherPopup(e);
    }

    private void afficherPopup(MouseEvent e) {
      if (e.isPopupTrigger()) {
        popup.show(e.getComponent(), e.getX(), e.getY());
      }
    }
  }

  /**
   * Classe incluse : listener sur les mouvements du curseur dans le
   * texte. Les boutons gras, italique et soulign sont mis  jour
   * en fonction du texte sur lequel se trouve le curseur. 
   */
  class CurseurTexteListener implements CaretListener {
    public void caretUpdate(CaretEvent e) {
      try {
        //On recupere le textPane  l'origine de l'evenement
        JTextPane source = (JTextPane)e.getSource();
        // On recupere les proprietes des caracteres au point
        // d'insertion courant
        AttributeSet attr = source.getCharacterAttributes();
        // On modifie les valeurs des actions en fonction de
        // l'etat du texte au point d'insertion
        actionGras.putValue(SELECTIONNE,
          new Boolean(StyleConstants.isBold(attr)));
        actionItalique.putValue(SELECTIONNE,
          new Boolean(StyleConstants.isItalic(attr)));
        actionSouligne.putValue(SELECTIONNE,
          new Boolean(StyleConstants.isUnderline(attr)));
      } catch (Exception ex) {
        // On peut avoir par exemple des ClassCastException
        ex.printStackTrace();
      }
    }
  }

  /**
   * Classe incluse : listener transformant la modification de la
   * propriete SELECTIONNE d'une action en la selection/deselection
   * du ToggleButton associe
   */
  class SelectionneListener implements PropertyChangeListener {
    protected AbstractButton bouton;
    public SelectionneListener(AbstractButton b) {
      bouton = b;
    }
    /** 
     * Methode appelee quand une propriete de l'objet ecoute
     * (l'Action) change. Seul le cas de la proprit
     * SELECTIONNE nous interesse.
     */
    public void propertyChange(PropertyChangeEvent e) {
      if (e.getPropertyName().equals(FenetreEditeur.SELECTIONNE)) {
        if (bouton != null) {
          bouton.setSelected(((Boolean)e.getNewValue()).booleanValue());
        }
      }
    }
  }

} // fin classe FenetreEditeur