package ihm;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.net.URL;
import applicatif.*;


import javax.swing.tree.*;

/**
 * Fentre principale de l'application. 
 * Cette fentre comprend un panneau divis en 2 parties : 
 * arbre des signets et description d'un signet. 
 * Elle comprend aussi une barre de menu et une barre d'tat. 
 * 
 * Cette classe est un singleton : elle possde au maximum
 * une seule instance. 
 */
 public class FenetrePrincipale extends JFrame {

  // Variables statiques
  private static final String TITRE = "Gestion de signets";
  private static final String SB_LOADING = "Chargement ...";
  private static final String SB_READY = "Prt";

  // Instance unique de la classe
  protected static FenetrePrincipale instance = null;

  // Layout de la fenetre
  protected BorderLayout layoutFenetre = new BorderLayout();
  // Split panel
  protected JSplitPane splitPanel = new JSplitPane();
  // Panneau de gauche affichant l'arbre des signets
  protected PanneauArbre panArbre;
  // Panneau de droite affichant le dtail d'un noeud
  protected PanneauDetail panDetail;
  // Barre d'tat
  protected JPanel barreEtat = new JPanel();
  // Layout de la barre d'tat
  protected GridLayout layoutBarreEtat = new GridLayout(1, 1);
  // JLabel pour afficher des messages dans la barre d'tat
  protected JLabel lbMsg = new JLabel(SB_LOADING);


  /** 
   * Cette mthode renvoie l'instance unique si elle existe
   * et la cre si ncessaire. 
   */
  public static FenetrePrincipale getInstance() {
    if (instance == null)  {
      instance = new FenetrePrincipale();
    }
    return instance;
  }

  /**
   * Constructeur protg
   */
  protected FenetrePrincipale() {
    // initialisation des variables d'instances
    panDetail = new PanneauDetail(this);
    panArbre = new PanneauArbre(this);
    // construction du split panel
    splitPanel.setLeftComponent(panArbre);
    splitPanel.setRightComponent(panDetail);
    splitPanel.setContinuousLayout(true);
    splitPanel.setOneTouchExpandable(true);
    // construction de la barre d'tat
    barreEtat.setLayout(layoutBarreEtat);
    barreEtat.add(lbMsg);
    // ajout des composants
    getContentPane().setLayout(layoutFenetre);
    getContentPane().add(splitPanel, BorderLayout.CENTER);
    getContentPane().add(barreEtat, BorderLayout.SOUTH);
    initialiserMenus();

    addWindowListener( new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        fermerApplication();
      }
    });
    pack();
    setTitle(TITRE);
  }

  public JTree getArbre() {
    return panArbre.getArbre();
  }
  
  public PanneauArbre getPanneauArbre() {
    return panArbre;
  }
  
  public PanneauDetail getPanneauDetail() {
    return panDetail;
  }
  

  /**
   * Cree et affecte les menus  la fentre. 
   * Le menu ouvrir lance un JFileChooser et fait appel
   * au GestionnaireNoeuds pour charger l'arbre  partir du fichier 
   * slectionn par l'utilisateur. 
   *   
   */
  protected void initialiserMenus() {
    JMenu menuFichier = new JMenu("Fichier");

    // menu ouvrir : pour charger depuis un fichier l'arbre de signets
    JMenuItem ouvrir = new   JMenuItem("Ouvrir");
    ouvrir.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent eve) {
        JFileChooser fc = new JFileChooser();
        int returnVal = fc.showOpenDialog(FenetrePrincipale.this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
          metier.GestionnaireNoeuds.getInstance().ouvrir(fc.getSelectedFile());
          ((DefaultTreeModel)getArbre().getModel()).reload();
        }
      }
    });
    menuFichier.add(ouvrir);


    // menu enregistrer : pour enregistrer dans un fichier l'arbre des signets
    JMenuItem enregistrer = new   JMenuItem("Enregistrer");
    enregistrer.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent eve) {
        JFileChooser fc = new JFileChooser();
        int returnVal = fc.showSaveDialog(FenetrePrincipale.this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
          metier.GestionnaireNoeuds.getInstance().enregistrer(fc.getSelectedFile());
          ((DefaultTreeModel)getArbre().getModel()).reload();
        }
      }
    });
    menuFichier.add(enregistrer);

    // menu quitter
    JMenuItem quitter = new   JMenuItem("Quitter");
    quitter.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent eve) {
        fermerApplication();
      }
    });
    menuFichier.addSeparator();
    menuFichier.add(quitter);


    JMenuBar mb = new JMenuBar();
    mb.add(menuFichier);
    setJMenuBar(mb);
  }

  public void show() {
    lbMsg.setText(SB_READY);
    super.show();
  }
  
  private void fermerApplication() {
    System.out.println("Merci d'avoir utilis l'application Signet");
    System.exit(0);
  }

}