package ihm;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.StyledDocument;
import java.net.*;
import java.io.*;
import metier.*;

/**
 * Cette classe reprsente le panneau situ dans la partie droite de la fentre
 * principale, qui affiche le dtail d'un noeud de l'arbre.
 */
public class PanneauDetail extends JPanel {

  // Constantes : 
  // 2 configurations possibles pour cette ihm suivant la nature
  // du noeud affich (catgorie ou signet)
  public static final int CONFIG_CATEGORIE = 0;
  public static final int CONFIG_SIGNET = 1;
  
    
  // Variables lies  l'IHM

  FenetrePrincipale fenetrePpale;
  JLabel lbType = new JLabel();
  JLabel lbNom = new JLabel();
  JLabel lbUrl = new JLabel();
  JLabel lbDescription = new JLabel();
  JRadioButton rbCat = new JRadioButton();
  JRadioButton rbSignet = new JRadioButton();
  ButtonGroup groupe = new ButtonGroup();
  JTextField tfNom = new JTextField();
  JTextField tfUrl = new JTextField();
  JScrollPane spDescription = new JScrollPane();
  JTextPane taDescription = new JTextPane();
  JButton btVoir = new JButton();
  JButton btEditer = new JButton();
  // Panels
  BorderLayout layout = new BorderLayout(3, 3);
  
  JPanel pHaut = new JPanel();
  BoxLayout lHaut = new BoxLayout(pHaut, BoxLayout.Y_AXIS);
  
  JPanel pBas = new JPanel();
  BorderLayout lBas = new BorderLayout();
  
  JPanel pLigne1 = new JPanel();
  BorderLayout lLigne1 = new BorderLayout();
  
  JPanel pLigne2 = new JPanel();
  BorderLayout lLigne2 = new BorderLayout();
  
  JPanel pLigne3 = new JPanel();
  BorderLayout lLigne3 = new BorderLayout();
  
  
  JPanel pDesc = new JPanel();
  FlowLayout lDesc = new FlowLayout(FlowLayout.LEFT);
  
  JPanel pType = new JPanel();
  FlowLayout lType = new FlowLayout(FlowLayout.LEFT);
  
  JPanel pBouton = new JPanel();
  FlowLayout lBouton = new FlowLayout(FlowLayout.CENTER);

  /**
   * Constructeur qui prend en paramtre la fentre "parent" de ce panneau.
   * Une rfrence sur cette fentre est ncessaier pour pouvoir ouvrir la
   * fentre modale d'dition.
   */
  public PanneauDetail(FenetrePrincipale fen) {
    fenetrePpale = fen;
    initialiserIHM();
    initialiserListener();
  }

  /** 
   * Cette mthode construit l'interface graphique
   */
  public void initialiserIHM() {
    lbType.setText("Type");
    lbNom.setText("Nom ");
    lbUrl.setText("URL ");
    lbDescription.setText("Description");
    rbCat.setText("Categorie");
    rbSignet.setText("Signet");
    groupe.add(rbCat);
    groupe.add(rbSignet);
    btVoir.setText("Voir URL");
    btEditer.setText("Editer description");

    // Le panel du haut :
    pHaut.setLayout(lHaut);
    pType.setLayout(lType);
    pType.add(rbCat);
    pType.add(rbSignet);

    pLigne1.setLayout(lLigne1);
    pLigne1.add(lbType, BorderLayout.WEST);
    pLigne1.add(pType,  BorderLayout.CENTER);
    //pLigne1.add(null, BorderLayout.EAST);
    
    
    pLigne2.setLayout(lLigne2);
    pLigne2.add(lbNom, BorderLayout.WEST);
    pLigne2.add(tfNom, BorderLayout.CENTER);
    //pLigne2.add(null, BorderLayout.EAST);
    
    pLigne3.setLayout(lLigne3);
    pLigne3.add(lbUrl, BorderLayout.WEST);
    pLigne3.add(tfUrl, BorderLayout.CENTER);
    pLigne3.add(btVoir, BorderLayout.EAST);
    
    pHaut.add(pLigne1);
    pHaut.add(pLigne2);
    pHaut.add(pLigne3);

    //  Le panel du bas :
    pBas.setLayout(lBas);
	taDescription.setEditable(false);
    spDescription.getViewport().add(taDescription);
    spDescription.setPreferredSize(new Dimension(350, 400));
    pBouton.setLayout(lBouton);
    pBouton.add(btEditer);
    pBouton.add(btVoir);
    pBas.add(pBouton, BorderLayout.SOUTH);
    pBas.add(spDescription, BorderLayout.CENTER);
    pDesc.setLayout(lDesc);
    pDesc.add(lbDescription);
    pBas.add(pDesc, BorderLayout.NORTH);
    
    // Le panel general
    this.setLayout(layout);
    this.add(pHaut, BorderLayout.NORTH);
    this.add(pBas, BorderLayout.CENTER);
    this.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
    pLigne1.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
    pLigne2.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
    pLigne3.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
    pBas.setBorder(BorderFactory.createEtchedBorder());
    pHaut.setBorder(BorderFactory.createEtchedBorder());
  }

  /**
   * Cette mthode gre les vnements. 
   */
  public void initialiserListener() {
    // ouverture de la fentre d'dition sur le clic sur "Editer"
    btEditer.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent eve) {
//        FenetreEditeur editeur = new FenetreEditeur(fenetrePpale, new DefaultStyledDocument());
        FenetreEditeur editeur = new FenetreEditeur(fenetrePpale, (DefaultStyledDocument)taDescription.getStyledDocument());
        editeur.show();
        repaint();
      }
    });
    
    // adaptation de l'IHM en focntion du radio bouton slectionn
    class ConfigListener implements ChangeListener {
      public void stateChanged(ChangeEvent e) {
        if (rbCat.isSelected()) {
          configurerIHM(CONFIG_CATEGORIE);
        }
        else {
          configurerIHM(CONFIG_SIGNET);
        }
      }
    }
    ConfigListener rbListener = new ConfigListener();
    rbCat.addChangeListener(rbListener);
    rbSignet.addChangeListener(rbListener);
    
    
    
    // ouverture de la fentre de visualisation HTML sur le clic sur "Voir"
    btVoir.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent eve) {
        URL url = null;
        try {
          url = new URL(tfUrl.getText());
        } catch (IOException e) {
          JOptionPane.showMessageDialog(PanneauDetail.this, 
            "Visualisation impossible : url introuvable", 
            "Visualisation impossible", JOptionPane.ERROR_MESSAGE);
        }
        FenetreHtml html = new FenetreHtml(fenetrePpale, url);
        html.show();
        repaint();
      }
    });

  }
  
  /**
   * Cette mthode permet de mettre  jour le texte correspondant
   *  la description. Elle est appele par exemple lorsqu'on sort de la 
   * fentre d'dition. 
   */
  public void setContenuDescription(StyledDocument doc/*String texte*/) {
    taDescription.setDocument(doc);
  }
  
  /**
   * Cette mthode permet d'adapter l'ihm suivant la nature du
   * noeud affich (affichage ou non du champ URL et du bouton Voir)
   */
  private void configurerIHM(int configuration) {
    if (configuration == CONFIG_CATEGORIE) {
      lbUrl.setVisible(false);
      tfUrl.setVisible(false);
      btVoir.setVisible(false);
    }
    else if (configuration == CONFIG_SIGNET) {
      lbUrl.setVisible(true);
      tfUrl.setVisible(true);
      btVoir.setVisible(true);
    }
  }
  
  /**
   * Cette mthode permet d'afficher les informations concernant
   * un noeud donn. 
   */
  public void afficherNoeud(Noeud noeud)  {
    if (noeud instanceof Categorie) {
      rbCat.setSelected(true);
      configurerIHM(CONFIG_CATEGORIE);
    }
    else if (noeud instanceof Signet) {
      rbSignet.setSelected(true);
      configurerIHM(CONFIG_SIGNET);
      URL urlNoeud = ((Signet)noeud).getUrl();
      if (urlNoeud != null) {
        tfUrl.setText(urlNoeud.toString());
      }
      else {
        tfUrl.setText("");
      }
    }
    tfNom.setText(noeud.getNom());
    taDescription.setText(noeud.getDescription());    
  }
  
  /**
   * Cette mthode permet d'enregistrer les informations
   * saisies dans l'objet mtier affich. 
   * 
   * @param noeud le noeud  enregistrer
   * @return le noeud modifi
   */
  public Noeud enregistrerNoeud(Noeud noeud)  {
    // Rcupration du champ nom. Si ce champ est vide, 
    // l'enregistrement ne peut avoir lieu. 
    String n = tfNom.getText();
    if (n == null || n.equals("")) {
      JOptionPane.showMessageDialog(this, 
        "Enregistrement impossible : un noeud doit avoir un nom", 
        "Enregistrement impossible", JOptionPane.ERROR_MESSAGE);
      return null;
    }
    // Cas o l'utilisateur a tranform la nature du noeud : 
    // signet -> catgorie ou inversement
    if (noeud instanceof Signet && rbCat.isSelected()) {
      noeud = new CategoriePersistante(n);
    }
    else if (noeud instanceof Categorie && rbSignet.isSelected()) {
      noeud = new Signet(n);
    }
    else {
      noeud.setNom(n);
      noeud.setDescription(taDescription.getText());
    }
    // Dans le cas d'un signet, on enregistre l'URL
    if (noeud instanceof Signet) {
      String url = tfUrl.getText();
      if (url != null) {
        try {
          ((Signet)noeud).setUrl(new URL(url));
        } catch (MalformedURLException e) {
			    System.out.println("L'URL est incorecte.");
        }
      }
    }
    return noeud;
  }
  
}