
package metier;

import java.util.ArrayList;
import java.util.Iterator;
import java.io.Serializable;

/**
 * Cette classe reprsente une catgorie dans laquelle on peut classer des signets
 * ou bien d'autres catgories. 
 * Cette classe porte l'information sur l'arborescence des signets. 
 * Autrement dit, une catgorie persistante permet d'accder  ses enfants. 
 * La catgorie persistante qui est la racine de l'arbre des signets sera 
 * srialise dans un fichier lors de l'enregistrement. 
 * Elle permettra de reconstruire l'arbre par dsrialisation lors de
 * l'ouverture. 
 */
public class CategoriePersistante extends Categorie implements Serializable {

  // Liste des enfants d'une catgorie : un enfant
  // peut tre un signet ou une autre catgorie
  protected ArrayList enfants = new ArrayList();

  /**
   * Construit une catgorie
   *
   * @param nom nom de la catgorie
   */
  public CategoriePersistante(String nom) {
    super(nom);
  }

  /**
   * Construit une catgorie
   *
   * @param nom nom de la catgorie
   * @param desc description de la catgorie
   */
  public CategoriePersistante(String nom, String desc) {
    super(nom, desc);
  }
  
  public void effaceEnfants() {
    enfants.clear();
  }
  
  public void addEnfant(Noeud n) {
    enfants.add(n);
  }
 
  public boolean sansEnfants() {
    return enfants.isEmpty();
  }
  
  public Iterator getEnfants() {
    return enfants.iterator();
  }
}