
package metier;

import javax.swing.JTree;
import javax.swing.tree.*;
import java.io.*;
import java.util.*;
import ihm.FenetrePrincipale;

/**
 * Cette classe permet de grer l'arborescence des signets.  
 */
public class GestionnaireNoeuds {
  
  // Instance unique de la classe
  // GestionnaireNoeud est un singleton
  private static GestionnaireNoeuds instance;
  protected CategoriePersistante racineM;
  protected DefaultMutableTreeNode racineG;

  /** 
   * Constructeur protg. 
   */
  protected GestionnaireNoeuds() {
  }
  
  /** 
   * Cette mthode renvoie l'instance unique si elle existe
   * et la cre si ncessaire. 
   */
  public static GestionnaireNoeuds getInstance() {
    if (instance == null)  {
      instance = new GestionnaireNoeuds();
    }
    return instance;
  }
  
  /**
   * Cette mthode enregistre une arborescence de signets dans le
   * fichier indiqu en paramtre. 
   * Pour cela, elle srialise l'objet CategoriePersistante racine
   * de l'arbre. 
   * 
   * @param fichier le fichier dans lequel on veut stocker l'arbre. 
   */
  public void enregistrer(File fichier) {
    JTree arbre = FenetrePrincipale.getInstance().getArbre();
    TreeModel model = arbre.getModel();
    if (model != null)  {
      racineG = (DefaultMutableTreeNode)model.getRoot();
      racineM = (CategoriePersistante)racineG.getUserObject();
      enregistrer(racineG, racineM);  
    }
    try {
      FileOutputStream ostream = new FileOutputStream(fichier);
      ObjectOutputStream p = new ObjectOutputStream(ostream);
      p.writeObject(racineM);
      p.flush();
      ostream.close();
    } catch (Exception e) {
      System.err.println("Une erreur lors de l'enregistrement de l'arbre "
        + "dans le fichier "+fichier. getName()+" est survenue. ");
      e.printStackTrace();
    }
  }
  
  /** 
   * Cette mthode parcourt les enfants d'un noeud donn,  
   * obtient les objets mtier associs et les ajoute
   *  la catgorie persistante passe en paramtre. 
   * 
   * @param pracineG le noeud parent (objet graphique)
   * @param pracineM la catgorie parent (objet mtier)
   */
  protected void enregistrer(DefaultMutableTreeNode pracineG, CategoriePersistante pracineM) {
    Enumeration enum = pracineG.children();
    DefaultMutableTreeNode nodeG;
    Noeud nodeM;
    pracineM.effaceEnfants();
    // Iteration sur les noeuds enfants du noeud pracineG
    while (enum.hasMoreElements())  {
      nodeG = (DefaultMutableTreeNode)enum.nextElement();
      nodeM = (Noeud)nodeG.getUserObject();
      // Ajout de l'objet mtier associ comme enfant de la catgorie
      // passe en paramtre
      pracineM.addEnfant(nodeM);
      if (!nodeG.isLeaf())  {
        enregistrer(nodeG, (CategoriePersistante)nodeM);
      }
    }
  }
  
  /**
   * Cette mthode charge une arborescence de signets depuis le
   * fichier indiqu en paramtre. 
   * Pour cela, elle dsrialise l'objet CategoriePersistante
   * stock dans le fichier, elle parcourt tous les objets metiers 
   * et construit au fur et  mesure les objets noeuds (graphiques)
   * 
   * @param fichier le fichier dans lequel a t stock la catgorie 
   * persistante racine.
   */
  public void ouvrir(File fichier) {
    // Dsrialisation de l'objet mtier stock dans le fichier
    try {
      FileInputStream istream = new FileInputStream(fichier);
      ObjectInputStream p = new ObjectInputStream(istream);
      racineM = (CategoriePersistante)p.readObject();
      istream.close();
    } catch (Exception e) {
      System.err.println("Une erreur dans la lecture du fichier "
        +fichier. getName()+" est survenue. ");
      e.printStackTrace();
    }
    // Construction du noeud racine
    racineG = new DefaultMutableTreeNode();
    racineG.setUserObject(racineM);
    ouvrir(racineG, racineM);
    JTree arbre = FenetrePrincipale.getInstance().getArbre();
    TreeModel model = new DefaultTreeModel(racineG);
    arbre.setModel(model);
  }
  
  /** 
   * Cette mthode parcourt les enfants d'une catgorie donne,  
   * construit les noeuds de l'arbre associs (objets graphiques) 
   * et les organise en arborescence. 
   * 
   * @param pracineG le noeud parent (objet graphique)
   * @param pracineM la catgorie parent (objet mtier)
   */
  protected void ouvrir(DefaultMutableTreeNode pracineG, CategoriePersistante pracineM) {
    Iterator ite = pracineM.getEnfants();
    DefaultMutableTreeNode nodeG;
    Noeud nodeM;
    // Iteration sur les enfants de la catgorie
    while (ite.hasNext())  {
      nodeM = (Noeud)ite.next();
      nodeG = new DefaultMutableTreeNode();
      nodeG.setUserObject(nodeM);
      pracineG.add(nodeG);
      CategoriePersistante categ;
      // Appel rcursif sur les sous-catgories 
      if (nodeM instanceof CategoriePersistante)  {
        categ = (CategoriePersistante)nodeM;
        if (!categ.sansEnfants())  {
          ouvrir(nodeG, (CategoriePersistante)nodeM);
        }
      }
    }
  }
}